/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2007 Ali Sabil <ali.sabil@gmail.com>
 *
 * Licensed under the GNU General Public License Version 2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gmodule.h>
#include <glib.h>
#include <string.h>
#include <pk-backend.h>
#include <pk-backend-spawn.h>

extern PkBackendSpawn *spawn;

/**
 * backend_get_groups:
 */
void
backend_get_groups (PkBackend *backend, PkEnumList *elist)
{
	g_return_if_fail (backend != NULL);
	pk_enum_list_append_multiple (elist,
				      PK_GROUP_ENUM_ACCESSORIES,
				      PK_GROUP_ENUM_GAMES,
				      PK_GROUP_ENUM_GRAPHICS,
				      PK_GROUP_ENUM_INTERNET,
				      PK_GROUP_ENUM_OFFICE,
				      PK_GROUP_ENUM_OTHER,
				      PK_GROUP_ENUM_PROGRAMMING,
				      PK_GROUP_ENUM_MULTIMEDIA,
				      PK_GROUP_ENUM_SYSTEM,
				      -1);
}

/**
 * backend_get_filters:
 */
void
backend_get_filters (PkBackend *backend, PkEnumList *elist)
{
	g_return_if_fail (backend != NULL);
	pk_enum_list_append_multiple (elist,
				      PK_FILTER_ENUM_GUI,
				      PK_FILTER_ENUM_INSTALLED,
				      PK_FILTER_ENUM_DEVELOPMENT,
				      -1);
}

/**
 * backend_get_description:
 */

void
backend_get_description (PkBackend *backend, const gchar *package_id)
{
        g_return_if_fail (backend != NULL);
        pk_backend_spawn_helper (spawn, "get-description.py", package_id, NULL);
}

/**
 * backend_search_details:
 */

void
backend_search_details (PkBackend *backend, const gchar *filter, const gchar *search)
{
        g_return_if_fail (backend != NULL);
        pk_backend_spawn_helper (spawn, "search-details.py", filter, search, NULL);
}

/**
 * backend_search_name:
 */
void
backend_search_name (PkBackend *backend, const gchar *filter, const gchar *search)
{
       g_return_if_fail (backend != NULL);
       pk_backend_spawn_helper (spawn, "search-name.py", filter, search, NULL);
}

/**
 * backend_search_group:
 */
void
backend_search_group (PkBackend *backend, const gchar *filter, const gchar *search)
{
        g_return_if_fail (backend != NULL);
        pk_backend_spawn_helper (spawn, "search-group.py", filter, search, NULL);
}

/* don't need to do any setup/finalize in the plain search mode */
void backend_init_search(PkBackend *backend) {}
void backend_finish_search(PkBackend *backend) {}
